package api

import (
	"encoding/json"
	"errors"
	"fmt"
	"sku3dweb/bus"
	"sku3dweb/db/model"

	"github.com/gin-gonic/gin"
	"github.com/xuri/excelize/v2"
	"infish.cn/comm"
)

func RegTreeAssetRemove(router *GinRouter) {
	router.POSTJWT("/tree/user/:assetType/r/:id", removeUserAsset)
	router.POSTJWT("/tree/team/:assetType/r/:id", removeTeamAsset)
	router.POSTJWT("/tree/company/:assetType/r/:id", removeCompanyAsset)

	router.GETJWT("/assets/user/download", DownloadUserAsset)
	router.POSTJWT("/assets/user/download", PostDownloadUserAsset)
	router.GETJWT("/assets/user/detail", detailUserAsset)
	router.GETJWT("/assets/team/detail", detailTeamAsset)
	router.GETJWT("/assets/company/detail", detailCompanyAsset)
	router.GET("/assets/platform/detail", detailPlatformAsset)
}

type DownLoadUserAssetRes struct {
	CompName    string `json:"compName"`
	MatName     string `json:"matName"`
	MatNum      string `json:"matNum"`
	CompanyName string `json:"companyName"`
}

func PostDownloadUserAsset(c *gin.Context, apictx *ApiSession) (interface{}, error) {
	var req []*DownLoadUserAssetRes
	err := c.BindJSON(&req)
	if err != nil {
		return nil, err

	}
	if len(req) < 1 {
		return nil, errors.New("没有数据")
	}
	f := excelize.NewFile()
	execl := NewProductBill(f)
	execl.Content = req
	execl.draws()
	c.Header("Content-Type", "application/octet-stream")
	c.Header("Content-Disposition", "attachment; filename="+"boom.xlsx")
	c.Header("Content-Transfer-Encoding", "binary")

	err = f.Write(c.Writer)
	if err != nil {
		return nil, err
	}

	return nil, nil
}

// !20240401迁移到 [POST] /assets/user/download,前端解析数据，后端只负责生成excel
// 验证后删除
func DownloadUserAsset(c *gin.Context, apictx *ApiSession) (interface{}, error) {
	sceneId := c.Query("sceneId")
	if len(sceneId) < 1 {
		return nil, errors.New("sceneId 不能为空")
	}
	result, err := detail(c, apictx, "user")
	if err != nil {
		return nil, err
	}
	j_res, err := json.Marshal(result)
	if err != nil {
		fmt.Println(err)
		return nil, err
	}
	asset := comm.AssetPackage{}
	err = json.Unmarshal(j_res, &asset)
	if err != nil {
		fmt.Println(err)
		return nil, err
	}

	// 获取场景产品
	sceneProds := []*comm.PackageSceneProduct{}
	scenes := asset.Source.Scenes
	for _, scene := range scenes {
		if scene.Id == sceneId {
			sceneProds = scene.Products
		}
	}
	// 获取products
	products := []*comm.PackageProduct{}
	for _, product := range asset.Source.Products {
		for _, sp := range sceneProds {
			if product.Id == sp.ProdId {
				products = append(products, product)
			}
		}

	}
	// 获取mats
	matInfos := []*DownLoadUserAssetRes{}
	for _, pd := range products {
		for _, pc := range pd.Components {
			for _, mat := range asset.Source.Mats {
				if mat.Id == pc.MatId {
					supplyName := "-"
					if userData, ok := pc.UserData.(map[string]interface{}); ok {
						if matSupply, ok := userData["matSupplyName"]; ok {
							if sname, ok := matSupply.(string); ok {
								supplyName = sname
							}
						}
					}
					matName := "-"
					matCusNum := "-"
					if mat.Name != "未命名" && len(mat.Name) > 0 {
						matName = mat.Name
					}
					if mat.CusNum != "未命名" && len(mat.CusNum) > 0 {
						matCusNum = mat.CusNum
					}
					matInfos = append(matInfos, &DownLoadUserAssetRes{
						CompName:    pc.Name,
						MatName:     matName,
						MatNum:      matCusNum,
						CompanyName: supplyName,
					})

				}

			}

		}
	}

	if len(matInfos) < 1 {
		return nil, errors.New("没有数据")
	}
	f := excelize.NewFile()
	execl := NewProductBill(f)
	execl.Content = matInfos
	execl.draws()
	c.Header("Content-Type", "application/octet-stream")
	c.Header("Content-Disposition", "attachment; filename="+"boom.xlsx")
	c.Header("Content-Transfer-Encoding", "binary")

	err = f.Write(c.Writer)
	if err != nil {
		return nil, err
	}

	return nil, nil
}

func detail(c *gin.Context, apictx *ApiSession, scope string) (interface{}, error) {
	assetType := c.Query("type")
	id := c.Query("id")
	if len(id) < 1 {
		return nil, fmt.Errorf("参数错误")
	}
	isCompany := false
	isSku3d := false
	if scope == SCOPE_PLATFORM {
		isSku3d = true
		isCompany = true
	} else {
		isSku3d = apictx.User.Parent == model.SKU3D_USERID
		isCompany = model.USERTYPE_COMPANY == apictx.User.UserType
	}
	srcColl := getAssetTypeCollectin(isCompany, isSku3d, scope, assetType)
	fmt.Println(srcColl)

	return bus.BusTreeAssetDetail(Sku3dDbName, srcColl, id)
}

func detailUserAsset(c *gin.Context, apictx *ApiSession) (interface{}, error) {
	return detail(c, apictx, "user")
}

func detailTeamAsset(c *gin.Context, apictx *ApiSession) (interface{}, error) {
	return detail(c, apictx, "team")
}

func detailCompanyAsset(c *gin.Context, apictx *ApiSession) (interface{}, error) {
	return detail(c, apictx, "company")
}
func detailPlatformAsset(c *gin.Context, apictx *ApiSession) (interface{}, error) {
	return detail(c, apictx, SCOPE_PLATFORM)
}

func remove(c *gin.Context, apictx *ApiSession, scope string) (interface{}, error) {
	assetType := c.Param("assetType")
	id := c.Param("id")
	if len(assetType) < 1 || len(id) < 1 {
		return nil, fmt.Errorf("参数错误")
	}

	isCompany := model.USERTYPE_COMPANY == apictx.User.UserType
	isSku3d := apictx.User.Parent == model.SKU3D_USERID

	srcColl := getAssetTypeCollectin(isCompany, isSku3d, scope, assetType)

	return true, bus.BusTreeAssetPackRemove(Sku3dDbName, srcColl, id)
}

func removeUserAsset(c *gin.Context, apictx *ApiSession) (interface{}, error) {

	return remove(c, apictx, "user")
}

func removeTeamAsset(c *gin.Context, apictx *ApiSession) (interface{}, error) {
	return remove(c, apictx, "team")
}

func removeCompanyAsset(c *gin.Context, apictx *ApiSession) (interface{}, error) {
	return remove(c, apictx, "company")
}
