package api

import (
	"github.com/gin-gonic/gin"
	"go.mongodb.org/mongo-driver/bson/primitive"
	"sku3dweb/db/model"
	"sku3dweb/db/repo"
)

//智能搭配接口
func RegCollocateRouter(router *GinRouter) {

	//获取项目搭配配置
	router.GETJWT("/collocate", func(c *gin.Context, apictx *ApiSession) (interface{}, error) {
		designId := c.Query("designId")
		scope := c.Query("scope")

		designIdObj, err := primitive.ObjectIDFromHex(designId)
		if err != nil {
			return nil, NewError("designId不能为空")
		}
		options := &repo.DocSearchOptions{Query: repo.Map{"designId": designIdObj, "scope": scope}, CollectName: repo.CollectionCollocates}

		collecate := &model.Collocate{}
		_, err = repo.RepoSeachDoc(apictx.CreateRepoCtx(), options, collecate)
		if collecate.Id == nil {
			return false, err
		}

		return collecate, err
	})

	//更新智能搭配配置
	router.POSTJWT("/collocate", func(c *gin.Context, apictx *ApiSession) (interface{}, error) {

		collocate := &model.Collocate{}
		err := c.ShouldBindJSON(collocate)
		if err != nil {
			return nil, err
		}

		if len(collocate.Comps) == 0 {
			return nil, NewError("请添加组件")
		}

		return repo.RepoUpsertSetDoc(apictx.CreateRepoCtx(), repo.CollectionCollocates, repo.Map{"designId": collocate.DesignId, "scope": collocate.Scope}, collocate)
	})

}
