package api

import (
	"context"
	"encoding/json"
	"errors"
	"fmt"
	"math/rand"
	"sku3dweb/db/model"
	"time"

	openapi "github.com/alibabacloud-go/darabonba-openapi/client"
	dysmsapi20170525 "github.com/alibabacloud-go/dysmsapi-20170525/v2/client"
	"github.com/alibabacloud-go/tea/tea"
	"github.com/go-redis/redis/v8"

	"github.com/gin-gonic/gin"
)

func createCode() string {
	return fmt.Sprintf("%06v", rand.New(rand.NewSource(time.Now().UnixNano())).Int31n(1000000))
}

/**
 * 使用AK&SK初始化账号Client
 * @param accessKeyId
 * @param accessKeySecret
 * @return Client
 * @throws Exception
 */
func createSmsClient() (_result *dysmsapi20170525.Client, _err error) {
	accessKeyId := tea.String("LTAI4FmvA9HNunGVq6biNASf")
	accessKeySecret := tea.String("QNHsxlW0iESK15TpouJwoLQzRMSxgu")

	config := &openapi.Config{
		// 您的AccessKey ID
		AccessKeyId: accessKeyId,
		// 您的AccessKey Secret
		AccessKeySecret: accessKeySecret,
	}
	// 访问的域名
	config.Endpoint = tea.String("dysmsapi.aliyuncs.com")
	//_result = &dysmsapi20170525.Client{}
	_result, _err = dysmsapi20170525.NewClient(config)
	return _result, _err
}

func SendSmsCode(phone string, code string) error {
	client, _err := createSmsClient()
	if _err != nil {
		return _err
	}

	sendSmsRequest := &dysmsapi20170525.SendSmsRequest{
		PhoneNumbers:  tea.String(phone),
		SignName:      tea.String("中鱼互动"),
		TemplateCode:  tea.String("SMS_158755156"),
		TemplateParam: tea.String(fmt.Sprintf("{\"code\":\"%s\"}", code)),
	}

	resp, err := client.SendSms(sendSmsRequest)
	if err != nil {
		return err
	}
	if *resp.Body.Code == "OK" {
		return nil
	}
	return errors.New(fmt.Sprintf("code err %s", *resp.Body.Code))
}

func SendSmsRegNotify(phone string, info *model.ContentInfo) error {
	client, _err := createSmsClient()
	if _err != nil {
		return _err
	}

	infobytes, _ := json.Marshal(info)

	sendSmsRequest := &dysmsapi20170525.SendSmsRequest{
		PhoneNumbers:  tea.String(phone),
		SignName:      tea.String("中鱼互动"),
		TemplateCode:  tea.String("SMS_232906383"),
		TemplateParam: tea.String(string(infobytes)),
	}

	resp, err := client.SendSms(sendSmsRequest)
	if err != nil {
		return err
	}
	if *resp.Body.Code == "OK" {
		return nil
	}
	return errors.New(fmt.Sprintf("code err %s", *resp.Body.Code))
}

func ServiceSmsSend(c *gin.Context, apictx *ApiSession) (interface{}, error) {

	body := struct {
		Phone string
		Use   string
	}{}
	err := c.ShouldBindJSON(&body)
	if err != nil {
		return nil, NewError("参数解析错误!")
	}

	if len(body.Phone) != 11 && len(body.Use) < 1 {
		return nil, NewError("参数错误!")
	}

	//查询缓存里面是否已经有验证码了
	key := fmt.Sprintf("sms#%s#%s", body.Phone, body.Use)

	ctx := context.Background()
	rdb := apictx.Svc.Redis
	val, err := rdb.Get(ctx, key).Result()
	if err != nil && err != redis.Nil {
		return nil, NewLogWithError(err)
	}

	code := val
	if len(val) < 1 {
		code = createCode()
	}

	err = SendSmsCode(body.Phone, code)
	if err != nil {
		return nil, err
	}
	rdb.Set(ctx, key, code, time.Duration(3*time.Minute))

	return true, nil
}
