package api

import (
	"sku3dweb/db/model"
	"sku3dweb/db/repo"
	"time"

	"github.com/gin-gonic/gin"
	"go.mongodb.org/mongo-driver/bson"
)

func CreateWebRouter(router *GinRouter) {

	//原创设计
	CreateCRUD(router, "/web/design", &CRUDOption{
		Collection: repo.CollectionInfoDesigns,

		NewModel: func(c *gin.Context) interface{} {
			body := &struct {
				Sex    uint8
				Type   string
				Season uint8
			}{}
			c.ShouldBindJSON(&body)

			nopublic := false

			defaultImg := &model.OssType{Url: "", Size: 0}
			scene := &model.InfoDesignScene{
				Name:   "111",
				Images: []*model.OssType{defaultImg},
			}

			return &model.InfoDesign{
				Name:     "原创设计",
				Sex:      body.Sex,
				Type:     body.Type,
				Season:   body.Season,
				IsPublic: &nopublic,
				Thumbnail: &model.OssType{
					Url:  "",
					Size: 0,
				},
				Scenes:     []*model.InfoDesignScene{scene},
				CreateTime: time.Now(),
			}
		},
		EmtyModel: func(*gin.Context) interface{} {
			return &model.InfoDesign{}
		},
		DetailProject: []string{"name", "thumbnail", "isPublic", "scenes", "weight", "createTime", "designId", "sex", "type", "season", "pv"},
		SearchProject: []string{"name", "thumbnail", "isPublic", "scenes", "weight", "createTime", "designId", "sex", "type", "season", "pv"},
		SearchSort:    bson.D{bson.E{"weight", 1}, bson.E{"createTime", -1}},
	})

	router.GET("/web/design/top", func(c *gin.Context, apictx *ApiSession) (interface{}, error) {
		_, size, _ := UtilQueryPageSize(c)

		return repo.RepoPageSearch(apictx.CreateRepoCtx(), &repo.PageSearchOptions{
			CollectName: repo.CollectionInfoDesigns,
			Sort:        bson.D{bson.E{"pv", -1}, bson.E{"_id", -1}},
			Size:        size,
			Page:        1,
		})
	})

	//流行趋势
	CreateCRUD(router, "/web/trend", &CRUDOption{
		Collection: repo.CollectionInfoTrends,

		NewModel: func(c *gin.Context) interface{} {

			body := &struct {
				Sex     uint8
				Type    string
				Season  uint8
				Content string
			}{}
			c.ShouldBindJSON(&body)

			nopublic := false

			return &model.InfoTrend{
				IsPublic: &nopublic,
				Sex:      body.Sex,
				Season:   body.Season,
				Type:     body.Type,
				Name:     "流行趋势",
				Content:  body.Content,
				Thumbnail: &model.OssType{
					Url:  "",
					Size: 0,
				},
				CreateTime: time.Now(),
			}
		},
		EmtyModel: func(*gin.Context) interface{} {
			return &model.InfoTrend{}
		},
		SearchProject: []string{"name", "thumbnail", "sex", "type", "season", "createTime"},
		DetailProject: []string{"name", "thumbnail", "sex", "type", "season", "content", "pv", "createTime"},
	})

	router.GET("/web/trend/top", func(c *gin.Context, apictx *ApiSession) (interface{}, error) {
		_, size, _ := UtilQueryPageSize(c)

		return repo.RepoPageSearch(apictx.CreateRepoCtx(), &repo.PageSearchOptions{
			CollectName: repo.CollectionInfoTrends,
			Sort:        bson.D{bson.E{"pv", -1}, bson.E{"_id", -1}},
			Size:        size,
			Page:        1,
		})
	})

	//env3d
	CreateCRUD(router, "/env3d", &CRUDOption{
		Collection: repo.CollectionEvn3d,
		NewModel: func(c *gin.Context) interface{} {
			body := &struct {
				Name      string
				Thumbnail *model.OssType
				Config    *model.Evn3dHdrConf
				Hdr       *model.OssType
			}{}
			c.ShouldBindJSON(&body)

			return &model.Env3d{
				Name:      body.Name,
				Thumbnail: body.Thumbnail,
				Config:    body.Config,
				HDR:       body.Hdr,
				Background: &model.Evn3dBackground{
					Type:  1,
					Color: &model.Vect3{0.749, 0.749, 0.749},
				},
				Options: &model.Env3dOption{
					Rotation: 0,
					Exposure: 1.3,
				},
				CreateTime: time.Now(),
			}
		},
		EmtyModel: func(*gin.Context) interface{} {
			return &model.Env3d{}
		},
		SearchProject: []string{"name", "config", "thumbnail", "hdr", "createTime", "options", "background", "toneMap"},
		DetailProject: []string{"name", "config", "thumbnail", "options", "background", "toneMap"},
	})
}
