package api

import (
	"fmt"
	_ "image/gif"
	_ "image/jpeg"
	_ "image/png"

	"github.com/xuri/excelize/v2"
)

type UserAssetDetailExcel struct {
	Row int

	Title string

	Excel *excelize.File

	SheetName string

	AlignCenterStyle int

	Content []*DownLoadUserAssetRes
}

func NewProductBill(f *excelize.File) *UserAssetDetailExcel {

	border := []excelize.Border{
		{Type: "top", Style: 1, Color: "000000"},
		{Type: "left", Style: 1, Color: "000000"},
		{Type: "right", Style: 1, Color: "000000"},
		{Type: "bottom", Style: 1, Color: "000000"},
	}

	sc, _ := f.NewStyle(&excelize.Style{
		Alignment: &excelize.Alignment{Horizontal: "center", Vertical: "center"},
		Border:    border,
		Font:      &excelize.Font{Size: 10},
	})

	b := &UserAssetDetailExcel{
		Title:            "BOM表单",
		SheetName:        "Sheet1",
		Excel:            f,
		AlignCenterStyle: sc,
	}
	marginLeft := 1.0
	b.Excel.SetPageMargins(b.SheetName, &excelize.PageLayoutMarginsOptions{
		Left: &marginLeft,
	})
	return b
}

func (b *UserAssetDetailExcel) drawTitle() error {
	b.Row++

	startCell := fmt.Sprintf("A%d", b.Row)
	endCell := fmt.Sprintf("D%d", b.Row)
	b.Excel.SetColWidth(b.SheetName, "A", "D", 16)
	err := b.Excel.MergeCell(b.SheetName, startCell, endCell)
	if err != nil {
		return err
	}

	style, err := b.Excel.NewStyle(&excelize.Style{
		Alignment: &excelize.Alignment{Horizontal: "center", Vertical: "center"},
		Font:      &excelize.Font{Bold: true, Size: 14}})
	if err != nil {
		return err
	}
	err = b.Excel.SetCellStyle(b.SheetName, startCell, startCell, style)
	if err != nil {
		return err
	}
	b.Excel.SetRowHeight(b.SheetName, b.Row, 23)
	b.Excel.SetCellValue(b.SheetName, startCell, b.Title)
	return nil
}

func (b *UserAssetDetailExcel) drawTableTitle() error {
	b.Row++
	// 部件名称 面料名称 面料编号 工厂信息
	var drawCol = func(prefix string, value string) error {
		left1Cell := fmt.Sprintf("%s%d", prefix, b.Row)
		err := b.Excel.SetCellStyle(b.SheetName, left1Cell, left1Cell, b.AlignCenterStyle)
		if err != nil {
			return err
		}
		b.Excel.SetRowHeight(b.SheetName, b.Row, 23)

		return b.Excel.SetCellValue(b.SheetName, left1Cell, value)
	}

	drawCol("A", "部件名称")
	drawCol("B", "面料名称")
	drawCol("C", "面料编号")
	drawCol("D", "工厂信息")
	return nil
}

func (b *UserAssetDetailExcel) drawTableContent() error {
	b.Row++
	var DrawRow = func(rowIndex int, values ...string) {
		charas := []string{"A", "B", "C", "D"}
		for i, c := range charas {
			v := ""
			if i < len(values) {
				v = values[i]
			}
			b.Excel.SetCellValue(b.SheetName, fmt.Sprintf("%s%d", c, rowIndex), v)
			val2Cel := fmt.Sprintf("%s%d", c, rowIndex)
			b.Excel.SetCellStyle(b.SheetName, val2Cel, val2Cel, b.AlignCenterStyle)

			b.Excel.SetRowHeight(b.SheetName, rowIndex, 21)
		}
	}
	booms := b.Content
	if len(booms) > 0 {
		for _, boom := range booms {
			DrawRow(b.Row, boom.CompName, boom.MatName, boom.MatNum, boom.CompanyName)
			b.Row++
		}
	}

	return nil
}

func (b *UserAssetDetailExcel) draws() {
	b.drawTitle()
	b.drawTableTitle()
	b.drawTableContent()
}
