package bus

import (
	"sku3dweb/db/repo"

	"infish.cn/comm"
)

type TreeDbDetailReq struct {
	DbName     string
	UserId     string
	AssetScope string
}

type TreeDbCategoryResp struct {
	Db                *comm.Database
	UserAssetCategory *comm.DbAssetUserCategory
	UserCategory      *comm.DbCategory
}

func BusGetDbDetail(UserId string) (*TreeDbCategoryResp, error) {
	out := &TreeDbCategoryResp{}
	err := NatsCenter.RequestPackApi("tree.db.category",
		&TreeDbDetailReq{
			DbName:     "sku3d-tree",
			UserId:     UserId,
			AssetScope: "sku3d",
		},
		out, &comm.RequestOptions{
			DeployPack: "queentree",
		})

	return out, err
}

type TreeListReq struct {
	DbName     string
	Collection string
	Page       int
	Size       int
	Project    []string
	Query      map[string]interface{}
}

func BusGetAssetList(req *TreeListReq) (*repo.PageResult, error) {
	ret := &repo.PageResult{}
	err := NatsCenter.RequestPackApi("tree.asset.list",
		req,
		ret, &comm.RequestOptions{
			DeployPack: "queentree",
		})
	return ret, err
}

func BusUpdateUserCategory(req *comm.DbCategory) (interface{}, error) {

	var ret interface{}
	err := NatsCenter.RequestPackApi("tree.user.category.update",
		req,
		&ret, &comm.RequestOptions{
			DeployPack: "queentree",
		})
	return ret, err
}

type UpdateDbCateReq struct {
	DbName string
	Cate   *comm.DbCategory
}

func BusUpdateDbCategory(DbName string, req *comm.DbCategory) error {

	err := NatsCenter.RequestPackApi("tree.db.category.update",
		&UpdateDbCateReq{
			DbName: DbName,
			Cate:   req,
		},
		nil, &comm.RequestOptions{
			DeployPack: "queentree",
		})
	return err
}

func BusUpdateUserAssetCategory(req *comm.DbAssetUserCategory) (interface{}, error) {

	var ret interface{}
	err := NatsCenter.RequestPackApi("tree.user.assetCategory.update",
		req,
		&ret, &comm.RequestOptions{
			DeployPack: "queentree",
		})
	return ret, err
}

type UserAssetCategoryReq struct {
	Scope  string
	UserId string
}

func BusGetUserAssetCategory(scope, userId string) (*comm.DbAssetUserCategory, error) {

	ret := &comm.DbAssetUserCategory{}

	err := NatsCenter.RequestPackApi("tree.user.assetCategory.query",
		&UserAssetCategoryReq{
			Scope:  scope,
			UserId: userId,
		},
		&ret, &comm.RequestOptions{
			DeployPack: "queentree",
		})

	return ret, err
}

type TreeDbReq struct {
	DbName string
}

func BusGetDbAssets(dbName string) (*comm.Database, error) {
	ret := &comm.Database{}
	err := NatsCenter.RequestPackApi("tree.db.assets",
		&TreeDbReq{
			DbName: dbName,
		},
		&ret, &comm.RequestOptions{
			DeployPack: "queentree",
		})
	return ret, err
}

type TreeDbUpdateReq struct {
	DbName string
	Db     *comm.Database
}

func BusUpdateDbAssets(dbName string, treeDb *comm.Database) error {

	return NatsCenter.RequestPackApi("tree.db.assets.update",
		&TreeDbUpdateReq{
			DbName: dbName,
			Db:     treeDb,
		},
		nil, &comm.RequestOptions{
			DeployPack: "queentree",
		})
}

type TreeAssetReq struct {
	DbName string
	Coll   string
	Id     string
}

// 3d模型资产包
func BusTreeAssetPackDetail(dbName string, coll string, id string) (*comm.AssetPackage, error) {
	out := &comm.AssetPackage{}
	return out, NatsCenter.RequestPackApi("tree.asset.scenepack.detail",
		&TreeAssetReq{
			DbName: dbName,
			Coll:   coll,
			Id:     id,
		},
		out, &comm.RequestOptions{
			DeployPack: "queentree",
		})
}

// 面料组详情
func BusTreeAssetMatgroupDetail(dbName string, coll string, id string) (*comm.AssetMatGroup, error) {
	out := &comm.AssetMatGroup{}
	return out, NatsCenter.RequestPackApi("tree.asset.matgroup.detail",
		&TreeAssetReq{
			DbName: dbName,
			Coll:   coll,
			Id:     id,
		},
		out, &comm.RequestOptions{
			DeployPack: "queentree",
		})
}

// 图片详情
func BusTreeAssetImageDetail(dbName string, coll string, id string) (*comm.AssetImage, error) {
	out := &comm.AssetImage{}
	return out, NatsCenter.RequestPackApi("tree.asset.image.detail",
		&TreeAssetReq{
			DbName: dbName,
			Coll:   coll,
			Id:     id,
		},
		out, &comm.RequestOptions{
			DeployPack: "queentree",
		})
}

func BusTreeAssetDetail(dbName string, coll string, id string) (map[string]interface{}, error) {
	out := &map[string]interface{}{}
	return *out, NatsCenter.RequestPackApi("tree.asset.detail",
		&TreeAssetReq{
			DbName: dbName,
			Coll:   coll,
			Id:     id,
		},
		out, &comm.RequestOptions{
			DeployPack: "queentree",
		})
}

type AssetInsertReq struct {
	DbName   string
	Coll     string
	Pack     *comm.AssetPackage
	Img      *comm.AssetImage
	Mat3d    *comm.AssetMatGroup
	OtherMat *comm.AssetMat
}

type AssetInsertResp struct {
	Id string
}

func BusTreeInsertAssetPack(dbName string, coll string, asset *AssetInsertReq) (string, error) {

	out := &AssetInsertResp{}
	return out.Id, NatsCenter.RequestPackApi("tree.asset.insert",
		&asset,
		out, &comm.RequestOptions{
			DeployPack: "queentree",
		})
}

func BusTreeAssetPackAdd(dbName string, coll string, asset *comm.AssetPackage) (string, error) {

	out := &AssetInsertResp{}
	return out.Id, NatsCenter.RequestPackApi("tree.asset.insert",
		&AssetInsertReq{
			DbName: dbName,
			Coll:   coll,
			Pack:   asset,
		},
		out, &comm.RequestOptions{
			DeployPack: "queentree",
		})
}

func BusTreeAssetMatgroupAdd(dbName string, coll string, asset *comm.AssetMatGroup) (string, error) {

	out := &AssetInsertResp{}
	return out.Id, NatsCenter.RequestPackApi("tree.asset.insert",
		&AssetInsertReq{
			DbName: dbName,
			Coll:   coll,
			Mat3d:  asset,
		},
		out, &comm.RequestOptions{
			DeployPack: "queentree",
		})
}

func BusTreeAssetImageAdd(dbName string, coll string, asset *comm.AssetImage) (string, error) {

	out := &AssetInsertResp{}
	return out.Id, NatsCenter.RequestPackApi("tree.asset.insert",
		&AssetInsertReq{
			DbName: dbName,
			Coll:   coll,
			Img:    asset,
		},
		out, &comm.RequestOptions{
			DeployPack: "queentree",
		})
}

func BusTreeAssetPackRemove(dbName string, coll string, id string) error {

	return NatsCenter.RequestPackApi("tree.asset.remove",
		&TreeAssetReq{
			DbName: dbName,
			Coll:   coll,
			Id:     id,
		},
		nil, &comm.RequestOptions{
			DeployPack: "queentree",
		})
}

type UpdateCommReq struct {
	Pack     *comm.AssetPackage
	Mat3d    *comm.AssetMatGroup
	OtherMat *comm.AssetMat
	Env3d    *comm.AssetEnv3dHdr
	Img      *comm.AssetImage
	Coll     string
	Db       string
}

type CreateShadowReq struct {
	Width int
	Scale int
	Id    string
	Coll  string
	Db    string
}

func BusTreeAssetUpdate(req *UpdateCommReq) error {
	return NatsCenter.RequestPackApi("tree.asset.update.header",
		req,
		nil, &comm.RequestOptions{
			DeployPack: "queentree",
		})
}

func BusTreeAssetReProcess(req *UpdateCommReq) error {
	return NatsCenter.RequestPackApi("tree.asset.pack.retreat",
		req,
		nil, &comm.RequestOptions{
			DeployPack: "queentree",
		})
}

// 创建阴影
func BusTreeAssetCreateShadow(req *CreateShadowReq) error {
	return NatsCenter.RequestPackApi("tree.asset.pack.reshadow",
		req,
		nil, &comm.RequestOptions{
			DeployPack: "queentree",
		})
}

func BusTreeAssetCreate(req *UpdateCommReq) (out interface{}, err error) {
	err = NatsCenter.RequestPackApi("tree.asset.create",
		req,
		&out, &comm.RequestOptions{
			DeployPack: "queentree",
		})
	return out, err
}

type TreeQueryDbCategoryReq struct {
	DefineCollection string
	DbName           string
	UserId           string
	AssetScope       string
}

func TreeQueryDbCategory(req *TreeQueryDbCategoryReq, cate *comm.DbCategory) error {
	return NatsCenter.RequestPackApi("tree.category.query", req, cate, &comm.RequestOptions{
		DeployPack: "queentree",
	})
}

type BusTreeSearchImageReq struct {
	DbName string
	Coll   string
	Result []*MlivusSearchRes
}

type MlivusSearchRes struct {
	ImHash    string  `json:"im_hash"`
	ProductId string  `json:"product_id"`
	Score     float64 `json:"score"`
}

func BusTreeSearchImage(req *BusTreeSearchImageReq) (out interface{}, err error) {
	err = NatsCenter.RequestPackApi("tree.search.image",
		req,
		&out, &comm.RequestOptions{
			DeployPack: "queentree",
		})
	return out, err
}
