package model

import (
	"time"

	"go.mongodb.org/mongo-driver/bson/primitive"
	"infish.cn/comm"
)

var (
	Applying     = 0
	ApplySuccess = 1
	ApplyFaild   = 2
	WatiPay      = 3
	SuccessPay   = 4
	FaildPay     = 5
	Trial         = 6
)

type Order struct {
	Id         *primitive.ObjectID `bson:"_id,omitempty" json:"_id,omitempty"`
	UserId     *primitive.ObjectID `bson:"userId,omitempty" json:"userId,omitempty"`
	UserName   string              `bson:"username,omitempty" json:"username,omitempty"`
	CreateTime time.Time           `bson:"createTime,omitempty" json:"createTime,omitempty"`
	UpdateTime time.Time           `bson:"updateTime,omitempty" json:"updateTime,omitempty"`
	Apply      *Apply              `bson:"apply,omitempty" json:"apply,omitempty"`
	Member     *MemberVo           `bson:"member,omitempty" json:"member,omitempty"`
	Status     *int                `bson:"status,omitempty" json:"status,omitempty"`
	// 0 支付宝 1微信
	PayMod *int `bson:"payMod,omitempty" json:"payMod,omitempty"`
	// 金额
	Amount *int `bson:"amount,omitempty" json:"amount,omitempty"`
}

type MemberVo struct {
	*Member
	CompanyName string `bson:"companyName,omitempty" json:"companyName,omitempty"`
	Phone       string `bson:"phone,omitempty" json:"phone,omitempty"`
}

// 会员申请
type Apply struct {
	Id         *primitive.ObjectID `json:"id,omitempty" bson:"id,omitempty"`
	UserId     *primitive.ObjectID `json:"userId,omitempty" bson:"userId,omitempty"`
	CreateTime time.Time           `bson:"createTime,omitempty" json:"createTime,omitempty"`
	UpdateTime time.Time           `bson:"updateTime,omitempty" json:"updateTime,omitempty"`

	Name       string `json:"name,omitempty" bson:"name,omitempty" binding:"required"`
	Phone      string `json:"phone,omitempty" bson:"phone,omitempty" binding:"required"`
	City       string `json:"city,omitempty" bson:"city,omitempty" binding:"required"`
	Enterprise string `json:"enterprise,omitempty" bson:"enterprise,omitempty"`
	// 企业所在地
	Address string `json:"address,omitempty" bson:"address,omitempty"`
	// 资料
	Materials []*comm.OssType `json:"materials,omitempty" bson:"materials,omitempty"`
	// 推荐设计师
	Designer string `json:"designer,omitempty" bson:"designer,omitempty"`
	// 推荐设计师手机
	DesignerPhone string `json:"designerPhone,omitempty" bson:"designerPhone,omitempty"`
	// 协会推荐
	Association string `json:"association,omitempty" bson:"association,omitempty"`
	// 附件
	Annexs []*comm.OssType `json:"annexs,omitempty" bson:"annexs,omitempty"`
}
